import * as lib_postgres from '@powersync/lib-service-postgres';
import { DisposableObserver } from '@powersync/lib-services-framework';
import { storage } from '@powersync/service-core';
import * as sync_rules from '@powersync/service-sync-rules';
import * as t from 'ts-codec';
import { CurrentDataDecoded } from '../../types/models/CurrentData.js';
import { RequiredOperationBatchLimits } from '../../types/types.js';
import { OperationBatch, RecordOperation } from './OperationBatch.js';
import { PostgresPersistedBatch } from './PostgresPersistedBatch.js';
export interface PostgresBucketBatchOptions {
    db: lib_postgres.DatabaseClient;
    sync_rules: sync_rules.SqlSyncRules;
    group_id: number;
    slot_name: string;
    last_checkpoint_lsn: string | null;
    no_checkpoint_before_lsn: string;
    store_current_data: boolean;
    keep_alive_op?: bigint | null;
    /**
     * Set to true for initial replication.
     */
    skip_existing_rows: boolean;
    batch_limits: RequiredOperationBatchLimits;
}
/**
 * Intermediate type which helps for only watching the active sync rules
 * via the Postgres NOTIFY protocol.
 */
declare const StatefulCheckpoint: t.Intersection<t.Codec<{
    id: number;
    last_checkpoint: bigint | null;
    last_checkpoint_lsn: string | null;
}, {
    id: number;
    last_checkpoint: string | number | null;
    last_checkpoint_lsn: string | null;
}, string, t.CodecProps>, t.ObjectCodec<{
    state: t.EnumCodec<typeof storage.SyncRuleState>;
}>>;
type StatefulCheckpointDecoded = t.Decoded<typeof StatefulCheckpoint>;
export declare class PostgresBucketBatch extends DisposableObserver<storage.BucketBatchStorageListener> implements storage.BucketStorageBatch {
    protected options: PostgresBucketBatchOptions;
    last_flushed_op: bigint | null;
    protected db: lib_postgres.DatabaseClient;
    protected group_id: number;
    protected last_checkpoint_lsn: string | null;
    protected no_checkpoint_before_lsn: string;
    protected persisted_op: bigint | null;
    protected write_checkpoint_batch: storage.CustomWriteCheckpointOptions[];
    protected readonly sync_rules: sync_rules.SqlSyncRules;
    protected batch: OperationBatch | null;
    private lastWaitingLogThrottled;
    constructor(options: PostgresBucketBatchOptions);
    get lastCheckpointLsn(): string | null;
    save(record: storage.SaveOptions): Promise<storage.FlushedResult | null>;
    truncate(sourceTables: storage.SourceTable[]): Promise<storage.FlushedResult | null>;
    protected truncateSingle(sourceTable: storage.SourceTable): Promise<bigint | null>;
    drop(sourceTables: storage.SourceTable[]): Promise<storage.FlushedResult | null>;
    flush(): Promise<storage.FlushedResult | null>;
    private flushInner;
    commit(lsn: string): Promise<boolean>;
    keepalive(lsn: string): Promise<boolean>;
    markSnapshotDone(tables: storage.SourceTable[], no_checkpoint_before_lsn: string): Promise<storage.SourceTable[]>;
    addCustomWriteCheckpoint(checkpoint: storage.BatchedCustomWriteCheckpointOptions): void;
    protected replicateBatch(db: lib_postgres.WrappedConnection, batch: OperationBatch): Promise<OperationBatch | null>;
    protected saveOperation(persistedBatch: PostgresPersistedBatch, operation: RecordOperation, currentData?: CurrentDataDecoded | null): Promise<{
        group_id: number;
        source_table: string;
        source_key: Buffer;
        data: Buffer;
        buckets: {
            id: string;
            bucket: string;
            table: string;
        }[];
        lookups: Buffer[];
    } | null>;
    /**
     * Gets relevant {@link SqlEventDescriptor}s for the given {@link SourceTable}
     * TODO maybe share this with an abstract class
     */
    protected getTableEvents(table: storage.SourceTable): sync_rules.SqlEventDescriptor[];
    protected withReplicationTransaction<T>(callback: (tx: lib_postgres.WrappedConnection) => Promise<T>): Promise<T>;
}
/**
 * Uses Postgres' NOTIFY functionality to update different processes when the
 * active checkpoint has been updated.
 */
export declare const notifySyncRulesUpdate: (db: lib_postgres.DatabaseClient, update: StatefulCheckpointDecoded) => Promise<void>;
export {};
