import * as lib_postgres from '@powersync/lib-service-postgres';
import { storage } from '@powersync/service-core';
import * as sync_rules from '@powersync/service-sync-rules';
import { models, RequiredOperationBatchLimits } from '../../types/types.js';
export type SaveBucketDataOptions = {
    /**
     * This value will be serialized into a BSON Byte array for storage
     */
    source_key: storage.ReplicaId;
    table: storage.SourceTable;
    before_buckets: models.CurrentBucket[];
    evaluated: sync_rules.EvaluatedRow[];
};
export type SaveParameterDataOptions = {
    source_key: storage.ReplicaId;
    table: storage.SourceTable;
    evaluated: sync_rules.EvaluatedParameters[];
    existing_lookups: Buffer[];
};
export type DeleteCurrentDataOptions = {
    source_table_id: bigint;
    /**
     * ReplicaID which needs to be serialized in order to be queried
     * or inserted into the DB
     */
    source_key?: storage.ReplicaId;
    /**
     * Optionally provide the serialized source key directly
     */
    serialized_source_key?: Buffer;
};
export type PostgresPersistedBatchOptions = RequiredOperationBatchLimits & {
    group_id: number;
};
export declare class PostgresPersistedBatch {
    group_id: number;
    /**
     * Very rough estimate of current operations size in bytes
     */
    currentSize: number;
    readonly maxTransactionBatchSize: number;
    readonly maxTransactionDocCount: number;
    /**
     * Ordered set of bucket_data insert operation parameters
     */
    protected bucketDataInserts: models.BucketData[];
    protected parameterDataInserts: models.BucketParameters[];
    protected currentDataDeletes: Pick<models.CurrentData, 'group_id' | 'source_key' | 'source_table'>[];
    /**
     * This is stored as a map to avoid multiple inserts (or conflicts) for the same key
     */
    protected currentDataInserts: Map<string, models.CurrentData>;
    constructor(options: PostgresPersistedBatchOptions);
    saveBucketData(options: SaveBucketDataOptions): void;
    saveParameterData(options: SaveParameterDataOptions): void;
    deleteCurrentData(options: DeleteCurrentDataOptions): void;
    upsertCurrentData(options: models.CurrentDataDecoded): void;
    shouldFlushTransaction(): boolean;
    flush(db: lib_postgres.WrappedConnection): Promise<void>;
    protected flushBucketData(db: lib_postgres.WrappedConnection): Promise<void>;
    protected flushParameterData(db: lib_postgres.WrappedConnection): Promise<void>;
    protected flushCurrentData(db: lib_postgres.WrappedConnection): Promise<void>;
}
export declare function currentBucketKey(b: models.CurrentBucket): string;
