import { storage } from '@powersync/service-core';
import * as t from 'ts-codec';
export declare const SyncRules: t.ObjectCodec<{
    id: t.Codec<number, number, "pg_number", t.CodecProps>;
    state: t.EnumCodec<typeof storage.SyncRuleState>;
    /**
     * True if initial snapshot has been replicated.
     *
     * Can only be false if state == PROCESSING.
     */
    snapshot_done: t.IdentityCodec<t.CodecType.Boolean>;
    /**
     * The last consistent checkpoint.
     *
     * There may be higher OpIds used in the database if we're in the middle of replicating a large transaction.
     */
    last_checkpoint: t.Union<t.Codec<null, null, string, t.CodecProps>, t.Codec<bigint, string | number, string, t.CodecProps>>;
    /**
     * The LSN associated with the last consistent checkpoint.
     */
    last_checkpoint_lsn: t.Union<t.Codec<null, null, string, t.CodecProps>, t.IdentityCodec<t.CodecType.String>>;
    /**
     * If set, no new checkpoints may be created < this value.
     */
    no_checkpoint_before: t.Union<t.Codec<null, null, string, t.CodecProps>, t.IdentityCodec<t.CodecType.String>>;
    slot_name: t.IdentityCodec<t.CodecType.String>;
    /**
     * Last time we persisted a checkpoint.
     *
     * This may be old if no data is incoming.
     */
    last_checkpoint_ts: t.Union<t.Codec<null, null, string, t.CodecProps>, t.Codec<Date, string, string, t.CodecProps>>;
    /**
     * Last time we persisted a checkpoint or keepalive.
     *
     * This should stay fairly current while replicating.
     */
    last_keepalive_ts: t.Union<t.Codec<null, null, string, t.CodecProps>, t.Codec<Date, string, string, t.CodecProps>>;
    /**
     * If an error is stopping replication, it will be stored here.
     */
    last_fatal_error: t.Union<t.Codec<null, null, string, t.CodecProps>, t.IdentityCodec<t.CodecType.String>>;
    keepalive_op: t.Union<t.Codec<null, null, string, t.CodecProps>, t.Codec<bigint, string | number, string, t.CodecProps>>;
    content: t.IdentityCodec<t.CodecType.String>;
}>;
export type SyncRules = t.Encoded<typeof SyncRules>;
export type SyncRulesDecoded = t.Decoded<typeof SyncRules>;
