import * as pg_wire from '@powersync/service-jpgwire';
import { configFile } from '@powersync/service-types';
import * as t from 'ts-codec';
export * as models from './models/models-index.js';
export declare const MAX_BATCH_RECORD_COUNT = 2000;
export declare const MAX_BATCH_ESTIMATED_SIZE = 5000000;
export declare const MAX_BATCH_CURRENT_DATA_SIZE = 50000000;
export declare const BatchLimits: t.ObjectCodec<{
    /**
     * Maximum size of operations we write in a single transaction.
     */
    max_estimated_size: t.OptionalCodec<t.Codec<number, number, string, t.CodecProps>>;
    /**
     * Limit number of documents to write in a single transaction.
     */
    max_record_count: t.OptionalCodec<t.Codec<number, number, string, t.CodecProps>>;
}>;
export type BatchLimits = t.Encoded<typeof BatchLimits>;
export declare const OperationBatchLimits: t.Intersection<t.Codec<{
    max_estimated_size?: number | undefined;
    max_record_count?: number | undefined;
}, {
    max_estimated_size?: number | undefined;
    max_record_count?: number | undefined;
}, string, t.CodecProps>, t.ObjectCodec<{
    /**
     * Maximum size of size of current_data documents we lookup at a time.
     */
    max_current_data_batch_size: t.OptionalCodec<t.Codec<number, number, string, t.CodecProps>>;
}>>;
export type OperationBatchLimits = t.Encoded<typeof OperationBatchLimits>;
export declare const PostgresStorageConfig: t.Intersection<t.Codec<{
    type: string;
} & {
    type: "postgresql";
    id?: string | undefined;
    tag?: string | undefined;
    uri?: string | undefined;
    hostname?: string | undefined;
    port?: number | undefined;
    username?: string | undefined;
    password?: string | undefined;
    database?: string | undefined;
    sslmode?: "verify-full" | "verify-ca" | "disable" | undefined;
    cacert?: string | undefined;
    client_certificate?: string | undefined;
    client_private_key?: string | undefined;
    tls_servername?: string | undefined;
    reject_ip_ranges?: string[] | undefined;
    slot_name_prefix?: string | undefined;
}, {
    type: string;
} & {
    type: "postgresql";
    id?: string | undefined;
    tag?: string | undefined;
    uri?: string | undefined;
    hostname?: string | undefined;
    port?: string | number | undefined;
    username?: string | undefined;
    password?: string | undefined;
    database?: string | undefined;
    sslmode?: "verify-full" | "verify-ca" | "disable" | undefined;
    cacert?: string | undefined;
    client_certificate?: string | undefined;
    client_private_key?: string | undefined;
    tls_servername?: string | undefined;
    reject_ip_ranges?: string[] | undefined;
    slot_name_prefix?: string | undefined;
}, string, t.CodecProps>, t.ObjectCodec<{
    /**
     * Allow batch operation limits to be configurable.
     * Postgres has less batch size restrictions compared to MongoDB.
     * Increasing limits can drastically improve replication performance, but
     * can come at the cost of higher memory usage or potential issues.
     */
    batch_limits: t.OptionalCodec<t.Codec<{
        max_estimated_size?: number | undefined;
        max_record_count?: number | undefined;
    } & {
        max_current_data_batch_size?: number | undefined;
    }, {
        max_estimated_size?: number | undefined;
        max_record_count?: number | undefined;
    } & {
        max_current_data_batch_size?: number | undefined;
    }, string, t.CodecProps>>;
}>>;
export type PostgresStorageConfig = t.Encoded<typeof PostgresStorageConfig>;
export type PostgresStorageConfigDecoded = t.Decoded<typeof PostgresStorageConfig>;
export type RequiredOperationBatchLimits = Required<OperationBatchLimits>;
export type NormalizedPostgresStorageConfig = pg_wire.NormalizedConnectionConfig & {
    batch_limits: RequiredOperationBatchLimits;
};
export declare const normalizePostgresStorageConfig: (baseConfig: PostgresStorageConfigDecoded) => NormalizedPostgresStorageConfig;
export declare const isPostgresStorageConfig: (config: configFile.BaseStorageConfig) => config is PostgresStorageConfig;
