var __addDisposableResource = (this && this.__addDisposableResource) || function (env, value, async) {
    if (value !== null && value !== void 0) {
        if (typeof value !== "object" && typeof value !== "function") throw new TypeError("Object expected.");
        var dispose, inner;
        if (async) {
            if (!Symbol.asyncDispose) throw new TypeError("Symbol.asyncDispose is not defined.");
            dispose = value[Symbol.asyncDispose];
        }
        if (dispose === void 0) {
            if (!Symbol.dispose) throw new TypeError("Symbol.dispose is not defined.");
            dispose = value[Symbol.dispose];
            if (async) inner = dispose;
        }
        if (typeof dispose !== "function") throw new TypeError("Object not disposable.");
        if (inner) dispose = function() { try { inner.call(this); } catch (e) { return Promise.reject(e); } };
        env.stack.push({ value: value, dispose: dispose, async: async });
    }
    else if (async) {
        env.stack.push({ async: true });
    }
    return value;
};
var __disposeResources = (this && this.__disposeResources) || (function (SuppressedError) {
    return function (env) {
        function fail(e) {
            env.error = env.hasError ? new SuppressedError(e, env.error, "An error was suppressed during disposal.") : e;
            env.hasError = true;
        }
        var r, s = 0;
        function next() {
            while (r = env.stack.pop()) {
                try {
                    if (!r.async && s === 1) return s = 0, env.stack.push(r), Promise.resolve().then(next);
                    if (r.dispose) {
                        var result = r.dispose.call(r.value);
                        if (r.async) return s |= 2, Promise.resolve(result).then(next, function(e) { fail(e); return next(); });
                    }
                    else s |= 1;
                }
                catch (e) {
                    fail(e);
                }
            }
            if (s === 1) return env.hasError ? Promise.reject(env.error) : Promise.resolve();
            if (env.hasError) throw env.error;
        }
        return next();
    };
})(typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
});
import { dropTables } from '../../utils/db.js';
import { openMigrationDB } from '../migration-utils.js';
export const up = async (context) => {
    const env_1 = { stack: [], error: void 0, hasError: false };
    try {
        const { service_context: { configuration } } = context;
        const client = __addDisposableResource(env_1, openMigrationDB(configuration.storage), true);
        /**
         * Request an explicit connection which will automatically set the search
         * path to the powersync schema
         */
        await client.transaction(async (db) => {
            await db.sql `
      CREATE SEQUENCE op_id_sequence AS int8 START
      WITH
        1
    `.execute();
            await db.sql `
      CREATE SEQUENCE sync_rules_id_sequence AS int START
      WITH
        1
    `.execute();
            await db.sql `
      CREATE TABLE bucket_data (
        group_id integer NOT NULL,
        bucket_name TEXT NOT NULL,
        op_id bigint NOT NULL,
        CONSTRAINT unique_id PRIMARY KEY (group_id, bucket_name, op_id),
        op text NOT NULL,
        source_table TEXT,
        source_key bytea,
        table_name TEXT,
        row_id TEXT,
        checksum bigint NOT NULL,
        data TEXT,
        target_op bigint
      )
    `.execute();
            await db.sql `CREATE TABLE instance (id TEXT PRIMARY KEY) `.execute();
            await db.sql `
      CREATE TABLE sync_rules (
        id INTEGER PRIMARY KEY,
        state TEXT NOT NULL,
        snapshot_done BOOLEAN NOT NULL DEFAULT FALSE,
        last_checkpoint BIGINT,
        last_checkpoint_lsn TEXT,
        no_checkpoint_before TEXT,
        slot_name TEXT,
        last_checkpoint_ts TIMESTAMP WITH TIME ZONE,
        last_keepalive_ts TIMESTAMP WITH TIME ZONE,
        keepalive_op BIGINT,
        last_fatal_error TEXT,
        content TEXT NOT NULL
      );
    `.execute();
            await db.sql `
      CREATE TABLE bucket_parameters (
        id BIGINT DEFAULT nextval('op_id_sequence') PRIMARY KEY,
        group_id integer NOT NULL,
        source_table TEXT NOT NULL,
        source_key bytea NOT NULL,
        lookup bytea NOT NULL,
        --- Stored as text which is stringified with JSONBig
        --- BigInts are not standard JSON, storing as JSONB seems risky
        bucket_parameters text NOT NULL
      );
    `.execute();
            await db.sql `
      CREATE INDEX bucket_parameters_lookup_index ON bucket_parameters (group_id ASC, lookup ASC, id DESC)
    `.execute();
            await db.sql `
      CREATE INDEX bucket_parameters_source_index ON bucket_parameters (group_id, source_table, source_key)
    `.execute();
            await db.sql `
      CREATE TABLE current_data (
        group_id integer NOT NULL,
        source_table TEXT NOT NULL,
        source_key bytea NOT NULL,
        CONSTRAINT unique_current_data_id PRIMARY KEY (group_id, source_table, source_key),
        buckets jsonb NOT NULL,
        data bytea NOT NULL,
        lookups bytea[] NOT NULL
      );
    `.execute();
            await db.sql `
      CREATE TABLE source_tables (
        --- This is currently a TEXT column to make the (shared) tests easier to integrate
        --- we could improve this if necessary
        id TEXT PRIMARY KEY,
        group_id integer NOT NULL,
        connection_id integer NOT NULL,
        relation_id jsonb,
        schema_name text NOT NULL,
        table_name text NOT NULL,
        replica_id_columns jsonb,
        snapshot_done BOOLEAN NOT NULL DEFAULT FALSE
      )
    `.execute();
            await db.sql `CREATE INDEX source_table_lookup ON source_tables (group_id, table_name)`.execute();
            await db.sql `
      CREATE TABLE write_checkpoints (
        user_id text PRIMARY KEY,
        lsns jsonb NOT NULL,
        write_checkpoint BIGINT NOT NULL
      )
    `.execute();
            await db.sql `
      CREATE TABLE custom_write_checkpoints (
        user_id text NOT NULL,
        write_checkpoint BIGINT NOT NULL,
        sync_rules_id integer NOT NULL,
        CONSTRAINT unique_user_sync PRIMARY KEY (user_id, sync_rules_id)
      );
    `.execute();
        });
    }
    catch (e_1) {
        env_1.error = e_1;
        env_1.hasError = true;
    }
    finally {
        const result_1 = __disposeResources(env_1);
        if (result_1)
            await result_1;
    }
};
export const down = async (context) => {
    const env_2 = { stack: [], error: void 0, hasError: false };
    try {
        const { service_context: { configuration } } = context;
        const client = __addDisposableResource(env_2, openMigrationDB(configuration.storage), true);
        await dropTables(client);
    }
    catch (e_2) {
        env_2.error = e_2;
        env_2.hasError = true;
    }
    finally {
        const result_2 = __disposeResources(env_2);
        if (result_2)
            await result_2;
    }
};
//# sourceMappingURL=1684951997326-init.js.map