var __addDisposableResource = (this && this.__addDisposableResource) || function (env, value, async) {
    if (value !== null && value !== void 0) {
        if (typeof value !== "object" && typeof value !== "function") throw new TypeError("Object expected.");
        var dispose, inner;
        if (async) {
            if (!Symbol.asyncDispose) throw new TypeError("Symbol.asyncDispose is not defined.");
            dispose = value[Symbol.asyncDispose];
        }
        if (dispose === void 0) {
            if (!Symbol.dispose) throw new TypeError("Symbol.dispose is not defined.");
            dispose = value[Symbol.dispose];
            if (async) inner = dispose;
        }
        if (typeof dispose !== "function") throw new TypeError("Object not disposable.");
        if (inner) dispose = function() { try { inner.call(this); } catch (e) { return Promise.reject(e); } };
        env.stack.push({ value: value, dispose: dispose, async: async });
    }
    else if (async) {
        env.stack.push({ async: true });
    }
    return value;
};
var __disposeResources = (this && this.__disposeResources) || (function (SuppressedError) {
    return function (env) {
        function fail(e) {
            env.error = env.hasError ? new SuppressedError(e, env.error, "An error was suppressed during disposal.") : e;
            env.hasError = true;
        }
        var r, s = 0;
        function next() {
            while (r = env.stack.pop()) {
                try {
                    if (!r.async && s === 1) return s = 0, env.stack.push(r), Promise.resolve().then(next);
                    if (r.dispose) {
                        var result = r.dispose.call(r.value);
                        if (r.async) return s |= 2, Promise.resolve(result).then(next, function(e) { fail(e); return next(); });
                    }
                    else s |= 1;
                }
                catch (e) {
                    fail(e);
                }
            }
            if (s === 1) return env.hasError ? Promise.reject(env.error) : Promise.resolve();
            if (env.hasError) throw env.error;
        }
        return next();
    };
})(typeof SuppressedError === "function" ? SuppressedError : function (error, suppressed, message) {
    var e = new Error(message);
    return e.name = "SuppressedError", e.error = error, e.suppressed = suppressed, e;
});
import { framework } from '@powersync/service-core';
import { PostgresMigrationAgent } from '../migrations/PostgresMigrationAgent.js';
import { normalizePostgresStorageConfig } from '../types/types.js';
import { PostgresBucketStorageFactory } from './PostgresBucketStorageFactory.js';
export const PostgresTestStorageFactoryGenerator = (factoryOptions) => {
    return async (options) => {
        try {
            const env_1 = { stack: [], error: void 0, hasError: false };
            try {
                const migrationManager = new framework.MigrationManager();
                const BASE_CONFIG = {
                    type: 'postgresql',
                    uri: factoryOptions.url,
                    sslmode: 'disable'
                };
                const TEST_CONNECTION_OPTIONS = normalizePostgresStorageConfig(BASE_CONFIG);
                const migrationAgent = __addDisposableResource(env_1, factoryOptions.migrationAgent
                    ? factoryOptions.migrationAgent(BASE_CONFIG)
                    : new PostgresMigrationAgent(BASE_CONFIG), true);
                migrationManager.registerMigrationAgent(migrationAgent);
                const mockServiceContext = { configuration: { storage: BASE_CONFIG } };
                if (!options?.doNotClear) {
                    await migrationManager.migrate({
                        direction: framework.migrations.Direction.Down,
                        migrationContext: {
                            service_context: mockServiceContext
                        }
                    });
                }
                await migrationManager.migrate({
                    direction: framework.migrations.Direction.Up,
                    migrationContext: {
                        service_context: mockServiceContext
                    }
                });
                return new PostgresBucketStorageFactory({
                    config: TEST_CONNECTION_OPTIONS,
                    slot_name_prefix: 'test_'
                });
            }
            catch (e_1) {
                env_1.error = e_1;
                env_1.hasError = true;
            }
            finally {
                const result_1 = __disposeResources(env_1);
                if (result_1)
                    await result_1;
            }
        }
        catch (ex) {
            // Vitest does not display these errors nicely when using the `await using` syntx
            console.error(ex, ex.cause);
            throw ex;
        }
    };
};
//# sourceMappingURL=PostgresTestStorageFactoryGenerator.js.map