import * as lib_postgres from '@powersync/lib-service-postgres';
import { logger } from '@powersync/lib-services-framework';
import { SqlSyncRules } from '@powersync/service-sync-rules';
export class PostgresPersistedSyncRulesContent {
    db;
    slot_name;
    id;
    sync_rules_content;
    last_checkpoint_lsn;
    last_fatal_error;
    last_keepalive_ts;
    last_checkpoint_ts;
    current_lock = null;
    constructor(db, row) {
        this.db = db;
        this.id = Number(row.id);
        this.sync_rules_content = row.content;
        this.last_checkpoint_lsn = row.last_checkpoint_lsn;
        this.slot_name = row.slot_name;
        this.last_fatal_error = row.last_fatal_error;
        this.last_checkpoint_ts = row.last_checkpoint_ts ? new Date(row.last_checkpoint_ts) : null;
        this.last_keepalive_ts = row.last_keepalive_ts ? new Date(row.last_keepalive_ts) : null;
    }
    parsed(options) {
        return {
            id: this.id,
            slot_name: this.slot_name,
            sync_rules: SqlSyncRules.fromYaml(this.sync_rules_content, options)
        };
    }
    async lock() {
        const manager = new lib_postgres.PostgresLockManager({
            db: this.db,
            name: `sync_rules_${this.id}_${this.slot_name}`
        });
        const lockHandle = await manager.acquire();
        if (!lockHandle) {
            throw new Error(`Sync rules: ${this.id} have been locked by another process for replication.`);
        }
        const interval = setInterval(async () => {
            try {
                await lockHandle.refresh();
            }
            catch (e) {
                logger.error('Failed to refresh lock', e);
                clearInterval(interval);
            }
        }, 30_130);
        return (this.current_lock = {
            sync_rules_id: this.id,
            release: async () => {
                clearInterval(interval);
                return lockHandle.release();
            }
        });
    }
}
//# sourceMappingURL=PostgresPersistedSyncRulesContent.js.map